Unit FMain;

Interface

Procedure Main;

Implementation

Uses
  CRT,         { keyressed function for faster bails                       }
  DOS,         { for FileRec structure                                     }
  Graph,       { graphics routines                                         }
  Vectors,     { vector operations and Float declaration                   }
  FDecls,      { most global declarations                                  }
  GridFunc,    { grid operations                                           }
  ParsEval,    { expression parser and evaluator                           }
  Graphics,    { graphics routines                                         }
  GetCmd,      { command line parser                                       }
  FileIO,      { file load/save operations                                 }
  Print,       { screen printing routines                                  }
  SetGraph,    { set graphics mode                                         }
  GrText,      { text drawing routine                                      }
  QuadFrac,    { fractal stuff                                             }
  Animate,     { animation stuff                                           }
  Plasma,      { plasma surface precompute                                 }
  Timer;       { timing routine                                            }

Const
  YCoord       : Word    = 0; { current vertical text coordinate }

{---------------------------------------------------------------------------}

Procedure SwapFloat(Var A,B : Float);
{
  swap two float values
}
Var
  Temp : Float;
Begin
  Temp := A;
  A := B;
  B := Temp
End; { Procedure SwapFloat }

{---------------------------------------------------------------------------}

Procedure ShowGraphInfo;
{
  Display graph information onscreen
}
Begin

  { If actually showing this stuff }
  If not DrawInfo then begin

    { show what function this is }
    If ShowFunc then begin
      PrintXY(0,YCoord*(TextHeight('W')+1),FuncStr);
      Inc(YCoord)
    End; { If }

    { and over what range }
    If ShowRange then begin
      PrintXY(0,YCoord*(TextHeight('W')+1), XMinStr);
      Inc(YCoord);
      PrintXY(0,YCoord*(TextHeight('W')+1), YMinStr);
      Inc(YCoord);
      PrintXY(0,YCoord*(TextHeight('W')+1), XMaxStr);
      Inc(YCoord);
      PrintXY(0,YCoord*(TextHeight('W')+1), YMaxStr);
      Inc(YCoord)
    End; { If }

    { print title }
    PrintXY(GetMaxX div 2 - (Length(TitleString) * TextWidth('H')) div 2,
              GetMaxY - TextHeight('W'),TitleString)

  End; { If }

End; { Procedure ShowGraphInfo }

{---------------------------------------------------------------------------}

Procedure TweakLights;
{
  Tweak lights to make life easier on the other routines
}
Var
  I,J,
  ActiveLights : Integer;
Begin

  { determine how many lights are active }
  ActiveLights := 0;
  For I := 1 to MaxLights do
    If Lights[I].InUse then
      Inc(ActiveLights);

  { adjust lighting to get a sum of 1.0*intensity at Focus }
  For I := 1 to MaxLights do
    With Lights[I] do
      If InUse then
        If Bright[RColor] = 0.0 then
          For J := RColor to MaxComponent do
            Bright[J] := Intensity * SqrDistBetween(Posn,Focus) / ActiveLights

End; { Procedure TweakLights }

{---------------------------------------------------------------------------}

Procedure CheckDist;
{
  Make sure viewpoint is far enough away from center of grid
}
Var
  VpDist : Float;
Begin

  { find distance to center of grid }
  VpDist := DistBetween(Viewpoint,Focus);

  { check against grid edges }
  If not IgnoreViewPointRange then
    If (VpDist < XMin + Focus.X) or (VpDist < YMin + Focus.Y) or
       (VpDist < XMax - Focus.X) or (VpDist < YMax - Focus.X) then
      Abort('Viewpoint is too close to center of grid')

End; { Procedure CheckDist }

{---------------------------------------------------------------------------}

Procedure DefineMainFunction;
{
  Define the main function
}
Var
  Error,Posn : Integer;
Begin
  Parse(FuncStr,Error,Posn);
  If (Error <> NoError) and (Error < LastError) then
    Abort('Error defining default main function')
End; { Procedure DefineMainFunction }

{---------------------------------------------------------------------------}

Procedure ShowParms;
{
  Output program header
}
Var
  I,J : Integer;
  S : String;
Begin

  Writeln('Function: ':15,FuncStr);

  { output Julia set parameter, if drawing Julia set }
  S := '';
  For I := 1 to Length(FuncStr) do
    S := S + UpCase(FuncStr[I]);
  If Pos('JULIA',S) > 0 then
    Writeln('Set parameter: ':15, RealJuliaParm:0:7, ',', ImagJuliaParm:0:7);

  Writeln('Grid corners: ':15, 'min: ', XMinStr, ',', YMinStr);
  Writeln(              '':15, 'max: ', XMaxStr, ',', YMaxStr);

  With ViewPoint do
    Writeln('Viewpoint: ':15, X:0:3, ',', Y:0:3, ',', Z:0:3);

  Write('Rendering: ':15);
  If Dithered then
    Write('dithered ')
  Else If Shaded then
    Write('shading ')
  Else
    Write('solid ');
  If WireFrame then
    Writeln('wireframe');

  If Shaded or Dithered then begin
    Writeln(#10'Lighting- ');
    For I := 1 to MaxLights do
      With Lights[I] do
        If Bright[RColor] >= 0 then begin
          Write(I:3,': ');
          With Posn do
            Write('Posn: ',X:0:3,',',Y:0:3,',',Z:0:3);
          Write('  Color: ');
          For J := RColor to MaxComponent do begin
            Write(Bright[J]:0:3);
            If J <> MaxComponent then
              Write(',')
          End; { For }
          Writeln
        End { If }
  End; { If }

  If PrintImage then
    Writeln('Printer Type: ':15,PrinterNames[Printer]);

  Writeln

End; { Procedure ShowParms }

{---------------------------------------------------------------------------}

Procedure Main;
{
  The main program.  Put here because it lets Turbo Pascal compile and link
  faster on a slow machine.
}
Begin

  { make sure header goes through BIOS (avoid snow problems) }
  DirectVideo := False;

  { output program header }
  Writeln;
  Writeln('Function plotter                          ' +
          'Version ' + Version + ' (c) 1990 Joe Slayton'#10#10);

  { parse user command line and/or input file(s) }
  GetCommandLine;

  { check if the user wants to do a screen view file replay }
  If AnimLoad then begin

    { make sure we come up in 320x200 256 color mode }
    Shaded    := True;
    WireFrame := False;
    Dithered  := False;

    { turn on graphics }
    GraphicsOn;

    { play back the screen file }
    PlayScreenViewFile;

  End; { If }

  { ensure that at least one display option was chosen }
  If not WireFrame and not Shaded and not Dithered then
    WireFrame := True;

  { and make sure that a dithered plot will actually show up }
  If Dithered then
    If FillColor = BackGndColor then
      If WireFrameColor = BackGndColor then
        FillColor := 15 - BackGndColor
      Else
        FillColor := WireFrameColor;

  { make sure we know if we should use a Postscript printer }
  UsePostScript := PrintImage and (Printer = PostScript);

  { if a main expression wasn't defined, define one }
  If not MainFunctionDefined then
    DefineMainFunction;

  { make sure that we know if we are gonna save this file }
  SavingFile := (FileOper = Save) and (FileType = SurfData);

  { determine if we need to precompute the surface }
  PreCompute := PreCompute or SavingFile or DoPlasma;

  { determine focus point (center of evaluation grid) }
  If not FocusSet then begin
    Focus.X := (XMin + XMax) / 2;
    Focus.Y := (YMin + YMax) / 2;
    If FloatZ then
      Focus.Z := Grid[YGridSize shr 1]^[XGridSize shr 1]
    Else
      Focus.Z := 0.0
  End; { If }

  { if no font size has been set, set one to 1/10 the largest dimension }
  If FontSize <= 0 then begin
    FontSize := (XMax - XMin);
    If (YMax - YMin) > FontSize then
      FontSize := YMax - YMin;
    FontSize := FontSize / 10
  End; { If }

  { make lights work together }
  TweakLights;

  { show program parameters if required }
  If PreCompute then
    ShowParms;

  { if loading data file, check file header, then load surface, maybe }
  If FileOper = Load then begin

    { check file header }
    CheckFile(DataFileName);

    { load surface, if it's a data file }
    If ((FileType = SurfData) or (FileType = SurfView)) then begin
      Writeln('Loading surface data from file ',DataFileName);
      PreCompute := True;
      DoLoadFile(FileType,DataFileName)
    End { If }

  End; { If }

  { force grid to go from low to high }
  If XMin > XMax then
    SwapFloat(XMin,XMax);
  If YMin > YMax then
    SwapFloat(YMin,YMax);

  { adjust grid position to help avoid many common divide-by-zero errors }
  XMin := XMin + (XMax - XMin) / 1000000.0;
  YMin := YMin + (YMax - YMin) / 1000000.0;
  XMax := XMax + (XMax - XMin) / 1000000.0;
  YMax := YMax + (YMax - YMin) / 1000000.0;

  { bound surface size }
  If XGridSize > MaxGridSize then
    XGridSize := MaxGridSize;
  If YGridSize > MaxGridSize then
    YGridSize := MaxGridSize;

  { disassemble code if required }
  If DoDisa then
    Disassemble;

  { if data file not available, and not loading image file, compute surface }
  If PreCompute then begin

    { allocate grid memory }
    AllocGrid(XGridSize, YGridSize);

    { start timer }
    TimerStart;

    { do plasma generation if required }
    If DoPlasma then
      PlasmaCalc;

    { for each calculation frame we require, compute the grid }
    For CalcFrame := 1 to MaxCalcFrames do begin
      ComputeGrid(XGridSize, YGridSize, XMin, YMin, XMax, YMax);
    End; { For }

    { end timer }
    ComputeTime := TimerEnd

  End; { If }

  { don't try to draw image if user requested a bail }
  If (Ch <> #27) then begin

    { turn on graphics hardware, set up palettes, etc. }
    GraphicsOn;

    { load image file if required }
    If LoadingImage then
      DoLoadFile(FileType,DataFileName)

    { else display surface }
    Else begin

      { make sure viewpoint is far enough away }
      CheckDist;

      { if animating, do the animation }
      If DoAnimate then
        DoAnimation

      Else begin

        { display graph information }
        If not AnimSave then
          ShowGraphInfo;

        { start timer }
        TimerStart;

        { draw surface }
        DrawGrid(XGridSize, YGridSize, Xmin, YMin, XMax, YMax);

        { draw user text }
        DrawText(UserTextXPos, UserTextYPos, UserTextZPos,
                 UserTextXRot, UserTextYRot, UserTextZRot, UserText);

        { dump page from printer if required }
        EjectPage;

        { end timer }
        DisplayTime := TimerEnd;

        { save the view if required }
        If AnimSave then
          SaveScreenView;

        { redraw graph information in case some was overwritten }
        YCoord := 0;
        ShowGraphInfo;

        { save file if requested }
        If FileOper = Save then
          DoSaveFile(FileType,DataFileName);

      End; { Else }

    End; { Else }

    { if not aborted, }
    If Ch <> #27 then begin

      { wait for time-out or user to press a key }
      If DelayGiven then begin
        DelayCount := DelayMilli;
        Repeat
          Delay(1);
          Dec(DelayCount)
        Until (DelayCount = 0) or KeyPressed
      End { If }
      Else begin
        PrintXY(0,YCoord*(TextHeight('W')+1),'Press any key to continue');
        Ch := GetKey
      End { Else }

    End; { If }

    { change back to text mode }
    CloseGraph

  End; { If }

  { output stuff }
  SignOff(TimerOn and (Ch <> #27));

End; { Procedure Main }

End.
