
Unit GDrivers;

(****************************************************************************
**                                                                         **
**  Install drivers for video systems                                      **
**                                                                         **
****************************************************************************)


Interface
Uses
  Graph;       { for graphics stuff }

Const

  GraphDevice  : Integer = Detect;  { graph device to use }
  GraphMode    : Integer = Detect;  { graph mode to use }

  VideoModeOfs = $0100;
  VideoDevOfs  = 12;

  { video Device numbers }
  GDS04  = VideoDevOfs + 0;  { 4-bit  (16    color) EGA/VGA/SVGA modes }
  GDS08  = VideoDevOfs + 1;  { 8-bit  (256   color) MCGA/VGA/SVGA modes }
  GDLAST = VideoDevOfs + 2;  { last driver name }

  { video mode numbers }
  GM322004 = VideoModeOfs +  0;
  GM644804 = VideoModeOfs +  1;
  GM806004 = VideoModeOfs +  2;
  GM107604 = VideoModeOfs +  3;
  GM121004 = VideoModeOfs +  4;

  GM322008 = VideoModeOfs +  5;
  GM644808 = VideoModeOfs +  6;
  GM806008 = VideoModeOfs +  7;
  GM107608 = VideoModeOfs +  8;
  GM121008 = VideoModeOfs +  9;

  GMLAST   = VideoModeOfs + 10;

Function ParseGraphMode(S : String) : Integer;
Function ParseGraphDevice(S : String) : Integer;
Procedure InitGDriver;

Implementation

Uses
  FDecls;      { for Abort }

Const

  { Video Device information }
  DevTable : Array[VideoDevOfs..GDLAST - 1] of Record
    Name   : String[13];  { name of graphics mode }
    Dev    : Integer;
  End = (
    (Name : 'VID04'    ; Dev : GDS04),
    (Name : 'VID08'    ; Dev : GDS08)
  );

  { Video Device masks for mode requirements }
  MaskS08  = 1 shl GDS08;   { 8-bit  (256   color) SVGA modes }
  MaskS04  = 1 shl GDS04;   { 4-bit  (16    color) EGA/VGA/SVGA modes }

  ModeTable : Array[VideoModeOfs..GMLAST - 1] of Record
    Name   : String[13];  { name of graphics mode }
    Mode   : Integer;     { graphics mode number }
    Device : LongInt;     { and Device codes required to support it }
  End = (
    (Name : '320X200X16'    ; Mode : GM322004; Device : MaskS04),
    (Name : '640X480X16'    ; Mode : GM644808; Device : MaskS04),
    (Name : '800X600X16'    ; Mode : GM806004; Device : MaskS04),
    (Name : '1024X768X16'   ; Mode : GM107604; Device : MaskS04),
    (Name : '1280X1024X16'  ; Mode : GM121004; Device : MaskS04),

    (Name : '320X200X256'   ; Mode : GM322008; Device : MaskS08),
    (Name : '640X480X256'   ; Mode : GM644808; Device : MaskS08),
    (Name : '800X600X256'   ; Mode : GM806008; Device : MaskS08),
    (Name : '1024X768X256'  ; Mode : GM107608; Device : MaskS08),
    (Name : '1280X1024X256' ; Mode : GM121008; Device : MaskS08)

  );

{---------------------------------------------------------------------------}

Function ParseGraphMode(S : String) : Integer;
{
  Parses the string S and returns the video mode specified by it.
  Returns 0 (Detect) if the mode was not in the name table
}
Var
  I : Integer;
Begin

  { check labels }
  I := VideoModeOfs;
  While (I < GMLAST) and (ModeTable[I].Name <> S) do
    Inc(I);

  { set proper mode and flag if found in any table }
  If I < GMLAST then
    ParseGraphMode := ModeTable[I].Mode
  Else
    ParseGraphMode := Detect;

End; { Function ParseGraphMode }

{---------------------------------------------------------------------------}

Function ParseGraphDevice(S : String) : Integer;
{
  Parses the string S and returns the video device specified by it.
  Returns 0 (Detect) if the mode was not in the name table
}
Var
  I : Integer;
Begin

  { check labels }
  I := VideoDevOfs;
  While (I < GDLAST) and (DevTable[I].Name <> S) do
    Inc(I);

  { set proper mode and flag if found in any table }
  If I < GDLAST then
    ParseGraphDevice := DevTable[I].Dev

  { check for some orphan devices }
  Else begin

    If (S = 'VGA') or (S = 'SVGA') then
      ParseGraphDevice := GDS04
    Else If S = 'SVGA256' then
      ParseGraphDevice := GDS08

    { not found, assume the system can handle a default device }
    Else
      ParseGraphDevice := Detect;

  End; { Else }

End; { Function ParseGraphDevice }

{---------------------------------------------------------------------------}

{$F+}

{ link in the video drivers }
Procedure SVGA16DriverProc;  External;  {$L SVGA16.OBJ  } { 16-col SVGA     }
Procedure SVGA256DriverProc; External;  {$L SVGA256.OBJ } { 256-col SVGA    }

{---------------------------------------------------------------------------}

Procedure InitGDriver;
{
  Turn on the graphics driver.  Bombs with an error if one occurs during
  initialization
}
Var
  I,
  GD, GM : Integer;
  GMask  : LongInt;

  Procedure Error(S : String);
  Begin
    Abort('Unable to register ' + S + ' video driver!');
  End; { Procedure Error }

Begin

(*
  Writeln('GraphDevice = ', GraphDevice, '     GraphMode = ', GraphMode - VideoModeOfs);
  Readln;
*)

  { If user specified a video mode but not a device }
  If (GraphMode <> Detect) and (GraphDevice = Detect) then begin

    { translate desired video mode to BGI mode number }
    Case GraphMode of

      GM322004 : GraphDevice := GDS04;
      GM644804 : GraphDevice := GDS04;
      GM806004 : GraphDevice := GDS04; 
      GM107604 : GraphDevice := GDS04;

      GM322008 : GraphDevice := GDS08;
      GM644808 : GraphDevice := GDS08; 
      GM806008 : GraphDevice := GDS08; 
      GM107608 : GraphDevice := GDS08;
      GM121008 : GraphDevice := GDS08;

    End; { Case }

  End { If }

  { If the user didn't specify a video device }
  Else If GraphDevice = Detect then begin

    { find out what kind of video adapter and recommended video mode }
    DetectGraph(GD, GM);

    { make a mask of the current video device }
    Case GD of

                 { all SVGAs test as VGAs to the default autodetect logic }
      VGA      : Begin
                   GMask := MaskS04 + MaskS08;
                   GraphDevice := GDS08
                 End;

      { we don't know what it is, tell the user about the problem }
      Else
        Abort('Unsupported Device!');

    End; { Case }

  End { If }
  { specified a video device, make a mask of it }
  Else
    GMask := 1 shl GraphDevice;

  { register the appropriate device driver and set up BGI device for init }
  { and also set the required default graphics mode if necessary }
  Case GraphDevice of

    VGA,
    GDS04  : Begin
               GD := InstallUserDriver('SVGA16', Nil);
               If RegisterBGIdriver(@SVGA16DriverProc) < 0 then
                 Error('16-color');
               If GraphMode = Detect then
                 GraphMode := GM644804;
             End;

    GDS08  : Begin
               GD := InstallUserDriver('SVGA256', Nil);
               If RegisterBGIdriver(@SVGA256DriverProc) < 0 then
                 Error('256-color');
               If GraphMode = Detect then
                 GraphMode := GM644808;
             End;

    Else
      Abort('Attempt to register unknown video driver (' + Int2Str(GraphDevice) + ')!');

  End; { Case }

  { translate desired video mode to BGI mode number }
  GM := 0;
  Case GraphMode of

    GM322004 : GM := 0; 
    GM644804 : GM := 3; 
    GM806004 : GM := 4;
    GM107604 : GM := 5;
    GM121004 : GM := 6;

    GM322008 : GM := 0;
    GM644808 : GM := 2;
    GM806008 : GM := 3;
    GM107608 : GM := 4;
    GM121008 : GM := 6;

  End; { Case }

(*
  Writeln('GraphDevice = ', GraphDevice, '   GraphMode = ', GraphMode - VideoModeOfs);
  Writeln('GD = ', GD, '   GM = ', GM);
  Writeln('Press any key to continue...');
  Readln;
*)
  { init graphics device driver }
  InitGraph(GD, GM, '');

  { bomb if graphics error }
  GD := GraphResult;
  If GD < 0 then
    Abort(GraphErrorMsg(GD));

End;  { Procedure InitGDriver }

{---------------------------------------------------------------------------}

End. { Unit GDrivers }
