Unit Graphics;

(****************************************************************************
**                                                                         **
**  Graphics display routines.                                             **
**                                                                         **
****************************************************************************)

Interface

Uses
  Graph,       { color constant values }
  Vectors,     { coordinate definitions }
  FDecls;      { abort routine and general declarations }

Function  LookupColor(ColorName : String) : Integer;

Procedure CalcColor(Xp, Yp, Zp  : Float;      { point for calcs }
                    Normal      : Vector;     { normal at point }
                    ViewDir     : Vector;     { viewing direction }
                    Var Color   : ColorType); { returned color at point }

Procedure DrawGrid(XGridSize, YGridSize : Word;
                   GridXMin, GridYMin,
                   GridXMax, GridYMax : Float);

Procedure DrawLine(X1,Y1, X2,Y2 : Float);


Implementation

Uses
  CRT,       { KeyPressed }
  Print,     { for printer type determinations }
  GrText,    { stroke text stuff }
  View,      { viewing transformation thingy }
  GDrivers,  { graphics drivers }
  Triangle,  { triangle drawing routine and associated fillers }
  ParsEval;  { evaluation routine }

Type

  FPointType = record
    X,Y : Single
  End; { Record }

  EdgeType = Array[0..MaxGridSize] of FPointType;

Var
  ViewDir       : Vector;      { facing toward viewpoint }
  Edge0, Edge1  : EdgeType;    { edge values }
  Z0, Z1        : GridLine;    { grid lines }

{---------------------------------------------------------------------------}

Function LookupColor;
{ look up color value in string given }

Const
  { search table for color parsing }
  ColorLookup : Array[0..15] of String[12] =
    ('BLACK',
     'BLUE',
     'GREEN',
     'CYAN',
     'RED',
     'MAGENTA',
     'BROWN',
     'LIGHTGRAY',
     'DARKGRAY',
     'LIGHTBLUE',
     'LIGHTGREEN',
     'LIGHTCYAN',
     'LIGHTRED',
     'LIGHTMAGENTA',
     'YELLOW',
     'WHITE');

Var
  I : Integer;
Begin

  { try to find given color in lookup table }
  I := 0;
  While (I <= 15) and (Pos(ColorName, ColorLookup[I]) <> 1) do
    Inc(I);

  { if color data was found, return value else return flag }
  If I <= 15 then
    LookupColor := I
  Else
    LookupColor := -1

End; { Procedure LookupColor }

{---------------------------------------------------------------------------}

Procedure CalcColor(Xp, Yp, Zp  : Float;      { point for calcs }
                    Normal  : Vector;         { normal at point }
                    ViewDir : Vector;         { viewing direction }
                    Var Color   : ColorType   { returned color at point }
);
{
  Gets the color of a given point using the global light database and
  viewing information.  Does not account for shadows.
  Xp, Yp, Zp are the current point to calculate the color for
}

Var
  CurLight     : Integer;   { current light under consideration }
  LComponent   : Integer;   { current light component being operated on }
  LightDir     : Vector;    { direction towards current light }
  H            : Vector;    { vector halfway between light and eye }
  EyeDir       : Vector;    { vector in direction of eye }
  Brightness   : ColorType; { specular brightness of point }
  SqrDist      : Float;     { square of distance to light source }
  DotProd      : Float;     { cos of angle from light to eye }
  WarnDotProd  : Float;     { cos of angle halfway between light and eye }
  PhongDotProd : Float;     { cos^n of angle between light and eye }
  Spec         : Float;     { Phong specular amount }

Begin

  { start with color equal to ambient lighting }
  Color := AmbientLight;

  { accumulate brightness and reflections from each active light }
  For CurLight := 1 to MaxLights do begin

    { if the light is active }
    If Lights[CurLight].InUse then begin

      { determine vector in direction of light from point }
      With Lights[CurLight] do
        MakeVector(Posn.X - Xp, Posn.Y - Yp, Posn.Z - Zp, LightDir);

      { determine square of distance between light and current point }
      SqrDist := sqr(LightDir.i) + sqr(LightDir.j) + sqr(LightDir.k);

      { make light source direction a unit vector }
      Unitize(LightDir);

      { determine visibility and diffuse brightness factor }
      DotProd := GetDotProduct(LightDir, Normal);

      { if it's a spotlight, adjust brightness of light source }
      With Lights[CurLight] do

        If IsSpot then begin

          { Make -LightDir }
          H.i := -LightDir.i;
          H.j := -LightDir.j;
          H.k := -LightDir.k;

          { get cos(angle from light to direction light is pointing) }
          WarnDotProd := GetDotProduct(H, Direction);

          { if it is in front of the light and within the cone, }
          If (WarnDotProd > 0) and (WarnDotProd > CosConeAngle) then begin

            { raise it to the beamwidth power }
            WarnDotProd := exp(SpotBWExp * ln(WarnDotProd));

            { and use to modify light intensity }
            For LComponent := RColor to MaxComponent do
              Brightness[LComponent] := Lights[CurLight].Bright[LComponent] * WarnDotProd;

          End { If }

          { spot is behind light, reset to 0.0 }
          Else
            For LComponent := RColor to MaxComponent do
              Brightness[LComponent] := 0.0;

        End { If }

        { not a spotlight, set brightness }
        Else
          Brightness := Lights[CurLight].Bright;

      { if this light shines on the patch, compute color }
      If (DotProd > 0) then begin

        { determine specular component }
        Spec := 0.0;

        { figure the highlight if required }
        If UseSpec then begin

          { determine unit vector halfway between light source and eye }
          MakeUnitVector(LightDir.i + ViewDir.i,
                         LightDir.j + ViewDir.j,
                         LightDir.k + ViewDir.k, H);

          { determine cos(angle between viewing direction and reflection) }
          PhongDotProd := GetDotProduct(H, Normal);

          { raise value to given power }
          If PhongDotProd > 0 then begin
            Spec := exp(PhongExp * ln(PhongDotProd))
          End; { If }

        End; { If }

        { accumulate diffuse and specular components, with distance effect }
        For LComponent := RColor to MaxComponent do begin
          Color[LComponent] := Color[LComponent] + SurfColor[LComponent] *
                               (Spec * SpecMulConst + DotProd * DiffMulConst) *
                               Brightness[LComponent] / SqrDist
        End; { For }

      End { If }

    End; { If }

  End;  { For }

  { determine vector in direction of eye from polygon corner }
  With ViewPoint do
    MakeUnitVector(X - Xp, Y - Yp, Z - Zp, EyeDir);

  { determine visibility }
  DotProd := GetDotProduct(EyeDir, Normal);

  { If the point isn't visible from the viewpoint, }
  If DotProd < 0 then begin

    { attenuate the color by the surface translucency factor }
    For LComponent := RColor to MaxComponent do
      Color[LComponent] := Color[LComponent] * Translucency[LComponent];

  End; { If }

  { make sure the color stays within bounds }
  For LComponent := RColor to MaxComponent do begin
    If Color[LComponent] < 0.0 then
      Color[LComponent] := 0.0;
    If Color[LComponent] > 1.0 then
      Color[LComponent] := 1.0
  End; { For }

End; { Procedure CalcColor }

{---------------------------------------------------------------------------}

Procedure LTransform(X, Y, Z : Float; Var SX, SY : Single);
{
  Scale the world-coordinate system point by the scale factors and then
  transform the point to a screen coordinate.
}
Begin

  { scale by the 3D scale factor }
  X := X * XScale;
  Y := Y * YScale;
  Z := Z * ZScale;

  { transfirm the 3D world coordinate to a 2D screen coordinate }
  If Transform(X, Y, Z, SX, SY) then

End; { Procedure LTransform }

{---------------------------------------------------------------------------}

Procedure DrawGrid;
{
  draw grid on screen
}
Var
  HalfGS,                  { half of grid size }
  Octant,                  { ocntant number }
  I,J,                     { loop counters for sort }
  IXMin,IYMin,             { starting index values }
  dIX,dIY,                 { change in index values }
  IX,IY      : Integer;    { index values }
  ViewAngle,               { angle from grid center to viewpoint }
  dX,dY,                   { XY grid mesh size }
  X,Y        : Float;      { current coordinate }
  Done       : Boolean;    { done drawing? }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

  Procedure Polygon(P0, P1, P2, P3 : FPointType;
                    x0, y0, z0,
                    x1, y1, z1,
                    x3, y3, z3 : Float);
  { draw (possibly colored) 3D polygon.
    Corners of polygon are: 0  3
                            1  2  note that corner 2 XYZ data is not passed.
    }
  Var
    Box      : Array[0..4] of PointType; { projected polygon corners         }
    Color    : ColorType; { shading value applied to patch                   }
    A, B,                 { sides of surface patch                           }
    Normal   : Vector;    { normal of surface patch                          }
    I,                    { current light number loop counter                }
    J,                    { count for light components                       }
    PMin, PMax : Integer; { min and max point for clip testing               }

  Begin

    { translate polygon corners to whole-number screen representation }
    Box[0].X := Round(PXMax shr 1 + P0.X - 0.5);
    Box[0].Y := Round(PYMax shr 1 + P0.Y - 0.5);
    Box[1].X := Round(PXMax shr 1 + P1.X - 0.5);
    Box[1].Y := Round(PYMax shr 1 + P1.Y - 0.5);
    Box[2].X := Round(PXMax shr 1 + P2.X - 0.5);
    Box[2].Y := Round(PYMax shr 1 + P2.Y - 0.5);
    Box[3].X := Round(PXMax shr 1 + P3.X - 0.5);
    Box[3].Y := Round(PYMax shr 1 + P3.Y - 0.5);
    Box[4] := Box[0];

    { find min and max X component }
    PMin := PXMax;
    PMax := -PXMax;
    For I := 0 to 3 do begin
      If Box[I].X < PMin then
        PMin := Box[I].X;
      If Box[I].X > PMax then
        PMax := Box[I].X;
    End; { For }

    { if both X coords off screen in same dir, return }
    If ((PMin < 0) and (PMax < 0)) or ((PMin > PXMax) and (PMax > PXMax)) then
      Exit;

    { find min and max Y component }
    PMin := PYMax;
    PMax := -PYMax;
    For I := 0 to 3 do begin
      If Box[I].Y < PMin then
        PMin := Box[I].Y;
      If Box[I].Y > PMax then
        PMax := Box[I].Y;
    End; { For }

    { if both Y coords off screen in same dir, return }
    If ((PMin < 0) and (PMax < 0)) or ((PMin > PYMax) and (PMax > PYMax)) then
      Exit;

    { determine polygon color, if required }
    If Dithered or Shaded then begin

      { make vectors down sides of patch }
      MakeVector((X3 - X0)*XScale, (Y3 - Y0)*YScale, (Z3 - Z0)*ZScale, A);
      MakeVector((X1 - X0)*XScale, (Y1 - Y0)*YScale, (Z1 - Z0)*ZScale, B);

      { determine unit normal to surface patch }
      GetCrossProduct(A, B, Normal);
      Unitize(Normal);

      { determine the color at the corner of the polygon }
      CalcColor(X0, Y0, Z0, Normal, ViewDir, Color);

    End; { If }

    { set color for inside of polygon }
    If Shaded then begin
      SetFillStyle(SolidFill, ColorBase + Round(Color[RColor] * 63));
      SetSolidFillPattern(ColorBase + Round(Color[RColor] * 63));
      If UsePostScript then
        If InverseHalf then
          PrintStr(Float2Str((1.0 - Color[RColor]) * 100,0) + ' ')
        Else
          PrintStr(Float2Str(Color[RColor] * 100,0) + ' ')
    End { If }

    Else If not Dithered then begin
      SetFillStyle(SolidFill, FillColor);
      SetSolidFillPattern(FillColor);
    End { Else If }

    Else If MaxComponent = RColor then begin

      { determine color (there are 65 halftones - 0 and 64 patterns) }
      I := Round(64 * Color[RColor]);

      { inverse halftone if desired }
      If InverseHalf then
        I := 64 - I;

      { set halftone pattern }
      SetFillPattern(FillPatternType(Halftones[I]), FillColor);

      { attempt using a simple RGB model }
      SetRGBFillPattern(Color);

      If UsePostScript then
        PrintStr(Float2Str(Color[RColor] * 100,0) + ' ');

    End { If }
    Else begin

      {

      This area reserved for dithered color value computation.  May require
      translation of color to HSI values to set dither pattern (as in the
      Triang1 program).

      }

      { attempt using a simple RGB model }
      SetRGBFillPattern(Color);

      If UsePostScript then
        PrintStr(Float2Str(Color[RColor] * 100,0) + ' ');

    End; { Else }

    { draw polygon (either system or custom version - system is faster) }
    FillTriangle(Box[0].X,Box[0].Y,Box[1].X,Box[1].Y,Box[3].X,Box[3].Y);
    FillTriangle(Box[1].X,Box[1].Y,Box[2].X,Box[2].Y,Box[3].X,Box[3].Y);

    { draw polygon (possibly edged) on PostScript printer }
    If UsePostScript then begin
      PrintStr(Float2Str(P0.X * 100,0) + ' ' + Float2Str(-P0.Y * 100,0) + ' ');
      PrintStr(Float2Str(P1.X * 100,0) + ' ' + Float2Str(-P1.Y * 100,0) + ' ');
      PrintStr(Float2Str(P2.X * 100,0) + ' ' + Float2Str(-P2.Y * 100,0) + ' ');
      PrintStr(Float2Str(P3.X * 100,0) + ' ' + Float2Str(-P3.Y * 100,0));
      PrintStr(' p')
    End; { If }

    { draw polygon edges for wireframe drawing }
    If WireFrame then begin

      SetColor(WireFrameColor);
      Line(Box[0].X,Box[0].Y,Box[1].X,Box[1].Y);
      Line(Box[1].X,Box[1].Y,Box[2].X,Box[2].Y);
      Line(Box[2].X,Box[2].Y,Box[3].X,Box[3].Y);
      Line(Box[3].X,Box[3].Y,Box[0].X,Box[0].Y);

    End; { If }

    If UsePostScript then
      PrintStr(#13#10)

  End; { Procedure Polygon }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

  Procedure ComputeYEdge(Var Edge : EdgeType; Var Z : GridLine);
  {
    compute an edge
  }
  Var
    I : Integer;
  Begin

    { compute major axis value }
    X := GridXMin + IX*dX;

    For I := 0 to YGridSize + 1 do begin

      { compute minor axis value }
      Y := GridYMin + I*dY;

      { determine value at the point }
      If PreCompute then
        Z[I] := Grid[IX]^[I]
      Else
        Z[I] := Evaluate(X,Y);

      { bound the value }
      If Z[I] < MinHeight then
        Z[I] := MinHeight
      Else If Z[I] > MaxHeight then
        Z[I] := MaxHeight;

      LTransform(X, Y, Z[I], Edge[I].X, Edge[I].Y)

    End { For }

  End; { Procedure ComputeYEdge }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

  Procedure ComputeXEdge(Var Edge : EdgeType; Var Z : GridLine);
  {
    compute an edge
  }
  Var
    I : Integer;
  Begin

    { compute major axis value }
    Y := GridYMin + IY*dY;

    For I := 0 to XGridSize + 1 do begin

      { compute minor axis value }
      X := GridXMin + I*dX;

      { determine value at the point }
      If PreCompute then
        Z[I] := Grid[I]^[IY]
      Else
        Z[I] := Evaluate(X,Y);

      { bound the value }
      If Z[I] < MinHeight then
        Z[I] := MinHeight
      Else If Z[I] > MaxHeight then
        Z[I] := MaxHeight;

      LTransform(X, Y, Z[I], Edge[I].X, Edge[I].Y)

    End { For }

  End; { Procedure ComputeXEdge }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

  Procedure DrawAxes;
  {
  }
  Var
    L,
    GridXSize, GridYSize,
    X,Y,Z : Float;
    SX1,SY1,
    SX2,SY2 : Single;
  Begin

    { draw X and Y axes }
    LTransform(GridXMin,GridYMin,0, SX1,SY1);
    LTransform(GridXMin,GridYMax,0, SX2,SY2);
    DrawLine(SX1,SY1, SX2,SY2);
    LTransform(GridXMax,GridYMin,0, SX2,SY2);
    DrawLine(SX1,SY1, SX2,SY2);

    { draw Z axis }
    LTransform(GridXMin,GridYMin,WallBase             , SX1,SY1);
    LTransform(GridXMin,GridYMin,WallBase + WallHeight, SX2,SY2);
    DrawLine(SX1,SY1, SX2,SY2);

    { figure axis label locations }
    GridXSize := GridXMax - GridXMin;
    GridYSize := GridYMax - GridYMin;

    X := GridXMin - dIX * GridXSize / 10;
    Y := GridYMin - dIY * GridYSize / 10;

    { draw axis labels }
    L := ((GridXMin + GridXMax) - dIX * TextLength(XAxisLabel)) / 2;
    DrawText(L,Y,0, 2.6  ,0,0,   XAxisLabel);

    L := ((GridYMin + GridYMax) + dIY * TextLength(YAxisLabel)) / 2;
    DrawText(X,L,0, 2.6  ,0,pi/2, YAxisLabel);

    L := TextLength(ZAxisLabel);
    DrawText(X    ,Y + L,0, 2.6,0,pi/4, ZAxisLabel);

  End; { Procedure DrawAxes }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

  Function IsContinuous(AtPoint:Integer; Var Line0,Line1:GridLine) : Boolean;
  {
    Check if the graph has a discontinuity at this point
  }
  Begin


(*

            | AtPoint
            v

       +    +    +       Line0

       +    +    +       Line1

       -  <--->  +


*)


    IsContinuous :=
       (abs(Line0[AtPoint] - Line0[AtPoint - 1]) <= Discontinuity) and
       (abs(Line0[AtPoint] - Line0[AtPoint + 1]) <= Discontinuity) and
       (abs(Line0[AtPoint] - Line1[AtPoint + 1]) <= Discontinuity) and
       (abs(Line0[AtPoint] - Line1[AtPoint + 0]) <= Discontinuity) and
       (abs(Line0[AtPoint] - Line1[AtPoint - 1]) <= Discontinuity)

  End; { Function IsContinuous }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

Const
  OctantTable : Array[1..8] of Boolean = (
    True,  { 1 }
    False, { 2 }
    False, { 3 }
    True,  { 4 }
    True,  { 5 }
    False, { 6 }
    False, { 7 }
    True   { 8 }
  );

Begin

  { init viewing vector }
  MakeUnitVector(Viewpoint.X, Viewpoint.Y, Viewpoint.Z, ViewDir);

  { determine grid step factors }
  dX := (GridXMax - GridXMin) / XGridSize;
  dY := (GridYMax - GridYMin) / YGridSize;

  { compute half of the grid size }
  HalfGS := Round(Sqrt(XGridSize*XGridSize + YGridSize*YGridSize)) shr 1;

  { determine if a "normal" view of data is OK (through quadrant 0 or 2)
    quadrants are calculated through corners of grid instead of side centers }

  { figure out viewing angle from center of grid to viewpoint (0 to 2*pi)}
  ViewAngle := atan2(ViewPoint.Y - Focus.Y,ViewPoint.X - Focus.X);
(*

  Start and step values for each starting angle (in octants)

                              -               +
                              <------ X ------>

               3*pi/4              2*pi/4               pi/4
                 135                 090                 045
                             Oct 3          Oct 2
                      \        Xs Max | Xs Min        /
                        \      Ys Min | Ys Min      /
      Oct 4               \    dX -1  | dX  1     /            Oct 1
                            \  dY  1  | dY  1   /
            + ^       Xs Max  +-------+-------+  Xs Min     ^ +
              |       Ys Min  | \  -+ | ++  / |  Ys Min     |
              |       dX -1   |   \   |   /   |  dX  1      |
                  1   dY  1   | -+  \ | /  ++ |  dY  1   0
              Y   8   --------+------ * ------+--------  0  Y
                  0   Xs Max  | --  / | \  +- |  Xs Min  0
              |       Ys Max  |   /   |   \   |  Ys Max     |
              |       dX  -1  | /  -- | +-  \ |  dX  1      |
            - v       dY  -1  +-------+-------+  dY -1      v -
                            /  Xs Max | Xs Min  \
      Oct 5               /    Ys Max | Ys Max    \            Oct 8
                        /      dX -1  | dX  1       \
                      /        dY -1  | dY -1         \
                             Oct 6          Oct 7
                 225                 270                 315
               5*pi/4              6*pi/4              7*pi/4

                              <----- X ------>
                              -               +

     Odd -numbered octants step in X (major - outer) then Y (minor - inner)
     Even-numbered octants step in Y (major - outer) then X (minor - inner)

     OctantNumber := Trunc(ViewAngle / (pi / 4)) + 1;

*)

  { determine which octant we are viewing from }
  Octant := Trunc(ViewAngle / (pi / 4)) + 1;
  If Octant < 0 then
    Octant := 4 - Octant;

  { determine start point and step values across grid }
  Case Octant of
    1, 2 : Begin
             IXMin := 0;
             IYMin := 0;
             dIX := +1;
             dIY := +1
           End;
    3, 4 : Begin
             IXMin := XGridSize;
             IYMin := 0;
             dIX := -1;
             dIY := +1
           End;
    5, 6 : Begin
             IXMin := XGridSize;
             IYMin := YGridSize;
             dIX := -1;
             dIY := -1
           End;
    7, 8 : Begin
             IXMin := 0;
             IYMin := YGridSize;
             dIX := +1;
             dIY := -1
           End;

  End; { Case }

  {!! draw axes first if required }
  If ShowAxes then
    DrawAxes;

  { we're never done before we start }
  Done := False;

  { draw correct version for each octant }
  If OctantTable[Octant] then begin

    { draw surface }
    IX := IXMin;

    { pre-compute edge }
    ComputeYEdge(Edge0, Z0);

    { draw rest of edges }
    Repeat

      { compute edge of polygon band }
      ComputeYEdge(Edge1, Z1);

      { make sure that the X coordinate is calculated }
      X := GridXMin + dX*IX;

      { draw polygon band }
      IY := IYMin;

      { skip segments to make circular, if required }
      If Circular then
        While sqr(HalfGS - IY) + sqr(HalfGS - IX) > sqr(HalfGS) do
          IY := IY + dIY;

      Repeat

        { figure the Y value (X value computed in ComputeEdge) }
        Y := GridYMin + IY*dY;

        { if the point is inside the grid }
        If (IY > 0) and (IY < YGridSize) then

(*
  It may be worth considering to put code in here that compares the polygon
  distance at each end of the current strip with the distance to the eye.
  The farther on ewould be drawn first.  This would continue until we reached
  the center of the screen
*)

          { if the function is continuous  }
          If IsContinuous(IY, Z0, Z1) then

            { draw the polygon }
            Polygon(Edge0[IY],Edge1[IY],Edge1[IY+1],Edge0[IY+1],
                    X   ,Y   ,Z0[IY],
                    X   ,Y+dY,Z0[IY+1],
                    X+dX,Y   ,Z1[IY]);

        { advance to the next position }
        IY := IY + dIY;

        { stop if we're doing circular  and within range }
        If Circular then
          Done := sqr(HalfGS - IY)+sqr(HalfGS - IX) > sqr(HalfGS);

      { go until off the either end of the polygon band }
      Until (IY < 0) or (IY >= YGridSize) or Done;

      { bump edge down one notch }
      Edge0 := Edge1;
      Z0 := Z1;
      IX := IX + dIX;

      { process user input, if required }
      If KeyPressed then
        Ch := GetKey

    { go until off the either end of the surface or user quits }
    Until (IX < 0) or (IX >= XGridSize) or (Ch = #27)

  End { If }
  Else begin

    { draw surface }
    IY := IYMin;

    { pre-compute edge }
    ComputeXEdge(Edge0, Z0);

    { draw rest of edges }
    Repeat

      { compute edge of polygon band }
      ComputeXEdge(Edge1, Z1);

      { make sure that the Y coordinate is calculated }
      Y := GridYMin + dY*IY;

      { draw polygon band }
      IX := IXMin;

      { skip segments to make circular, if required }
      If Circular then
        While sqr(HalfGS - IX) + sqr(HalfGS - IY) > sqr(HalfGS) do
          IX := IX + dIX;

      Repeat

        { figure the X value (Y value computed in ComputeEdge) }
        X := GridXMin + IX*dX;

        { draw correct polygon version }
        If (IX > 0) and (IX < XGridSize) then
          If IsContinuous(IY,Z0,Z1) then
            Polygon(Edge0[IX],Edge1[IX],Edge1[IX+1],Edge0[IX+1],
                    X   ,Y   ,Z0[IX],
                    X   ,Y+dY,Z0[IX+1],
                    X+dX,Y   ,Z1[IX]);

        IX := IX + dIX;

        Done := Circular and (sqr(HalfGS - IX)+sqr(HalfGS - IY) > sqr(HalfGS));

      { go until off the either end of the polygon band }
      Until (IX < 0) or (IX >= XGridSize) or Done;

      { bump edge down one notch }
      Edge0 := Edge1;
      Z0 := Z1;

      IY := IY + dIY;

      { process user input, if required }
      If KeyPressed then
        Ch := GetKey

    { go until off the either end of the surface or user quits }
    Until (IY < 0) or (IY >= YGridSize) or (Ch = #27)

  End { Else }

End; { Procedure DrawGrid }

{---------------------------------------------------------------------------}

Procedure DrawLine;
{
  draw a line to screen and to printer if required
}
Begin

  { draw the line on the screen }
  Line(PXMax shr 1 + Round(X1), PYMax shr 1 + Round(Y1),
       PXMax shr 1 + Round(X2), PYMax shr 1 + Round(Y2));

  { output line to printer if required }
  If UsePostScript then
    PrintStr(Float2Str(X2 * 100,0) + ' ' + Float2Str(-Y2 * 100,0) + ' ' +
             Float2Str(X1 * 100,0) + ' ' + Float2Str(-Y1 * 100,0) +
             ' l'#13#10)

End; { Procedure DrawLine }

{-------------------------------------------------------------------------}

{ main initialization code }

Const

  { 64 halftone patterns, in compressed form }
  HalftoneMask : Array[0..7,0..7] of Byte = (
    (00, 32, 08, 40, 02, 34, 10, 42),
    (48, 16, 56, 24, 50, 18, 58, 26),
    (12, 44, 04, 36, 14, 46, 06, 38),
    (60, 28, 52 ,20, 62, 30, 54, 22),
    (03, 35, 11, 43, 01, 33, 09, 41),
    (51, 19, 59, 27, 49, 17, 57, 25),
    (15, 47, 07, 39, 13, 45, 05, 37),
    (63, 31, 55, 23, 61, 29, 53, 21));

Var
  I,J,Pat : Integer; { used to unpack halftone table }
Begin

  { clear halftone pattern table }
  FillChar(Halftones,SizeOf(Halftones),0);

  { unpack halftone patterns }
  For Pat := 1 to 64 do
    For I := 0 to 7 do
      For J := 0 to 7 do
        If HalftoneMask[I][J] <= Pat - 1 then
          Halftones[Pat][I] := Halftones[Pat][I] or (1 shl J);

End. { Unit Graphics }
