
Unit GridFunc;

(****************************************************************************
**                                                                         **
**  Grid operations (except grid drawing, which is in GRAPHICS.PAS)        **
**                                                                         **
****************************************************************************)

Interface

Uses
  FDecls,
  Vectors;

Function GridMemSize(XSize, YSize : Word) : LongInt;
Function WillSurfaceFit(XSize, YSize : Word) : Boolean;
Procedure AllocGrid(Var XSize, YSize : Word);
Procedure ComputeGrid(XGridSize, YGridSize : Word;
                      GridXMin, GridYMin, GridXMax, GridYMax : Float);

Implementation

Uses
  CRT,        { KeyPressed }
  FileIO,     { file operation types }
  ParsEval;   { parser/evaluator }

{---------------------------------------------------------------------------}

Function GridMemSize(XSize, YSize : Word) : LongInt;
{
  Return amount of memory used by grid of given size
}
Begin
  GridMemSize := Round((Succ(XSize) * 1.0 * Succ(YSize)) * SizeOf(Grid[0]^[0]))
End; { Function GridMemSize }

{---------------------------------------------------------------------------}

Function WillSurfaceFit(XSize, YSize : Word) : Boolean;
{
  Determine if a surface of the given size will fit into memory
}
Begin
  WillSurfaceFit := GridMemSize(XSize, YSize) <= MemAvail - ReservedMem
End; { Function WillSurfaceFit }

{---------------------------------------------------------------------------}

Procedure AllocGrid(Var XSize, YSize : Word);
{
  Allocate grid memory and return size of grid actually allocated
}
Var
  I : Integer;
  MemUsed : Float;
Begin

  { determine a grid size that will fit into memory }
  If GridMemSize(XSize, YSize) > MemAvail - ReservedMem then begin

    Writeln('Grid too large to fit in memory, reducing...');

    I := 0;
    Repeat
      I := I + 1;
      If Odd(I) then
        Dec(YSize)
      Else
        Dec(XSize)
    Until (GridMemSize(XSize, YSize) <= MemAvail - ReservedMem) or
          (XSize < MinGridSize) or (YSize < MinGridSize);

    Writeln('Grid reduced to ', XSize, 'x', YSize, ' points.');

  End; { If }

  { make sure there is a good-sized grid to use }
  If (XSize < MinGridSize) or (YSize < MinGridSize) then
    Abort('A ' + Int2Str(XSize) + 'x' + Int2Str(YSize) +
          'grid is too small to calculate properly');

  { remember how much memory used by the surface }
  MemUsed := GridMemSize(XSize, YSize);

  { output memory status }
  Writeln(MemAvail, ' bytes free, ',
          MemUsed:0:0, ' bytes (',
          100 * MemUsed / MemAvail:0:2, '%) used by grid'#13#10,
          Succ(XSize), 'x', Succ(YSize), ' size grid'#10);

  { allocate memory for surface and set it all to 0.0 }
  For I := 0 to YSize do begin
    GetMem(Grid[I], SizeOf(Grid[0]^[0]) * Succ(XSize));
    FillChar(Grid[I]^, SizeOf(Grid[0]^[0]) * Succ(XSize), 0);
  End; { For }

End; { Function AllocGrid }

{---------------------------------------------------------------------------}

Procedure ComputeGrid(XGridSize, YGridSize : Word;
                      GridXMin, GridYMin, GridXMax, GridYMax : Float);
{
  Compute grid values
}
Var
  Y, X : Integer;
  XVal, YVal,
  GridXStep, GridYStep : Float;
  NPoints   : LongInt;
  MaxPoints : Float;
Begin

  { output the status format to the screen }
  Write(0.0:6:2,'% done computing surface points');

  { determine step factor across grid }
  GridXStep := (GridXMax - GridXMin) / (XGridSize + 1);
  GridYStep := (GridYMax - GridYMin) / (YGridSize + 1);

  { determine the current and maximum number of points calculated }
  NPoints := 0;
  MaxPoints := ((XGridSize + 1) * 1.0 * (YGridSize + 1)) / 100.0;

  { start the YVal position and index }
  YVal := GridYMin;
  Y := 0;

  Repeat

    { start the X position and index }
    XVal := GridXMin;

    { step through each X index value }
    For X := 0 to XGridSize do begin

      { evaluate the function for the current point }
      Grid[Y]^[X] := Grid[Y]^[X] + IEvaluate(XVal, YVal, X, Y);

      { update the X position  }
      XVal := XVal + GridXStep;

      { update the current number of points }
      NPoints := NPoints + 1;

      { output a status message every few points }
      If (NPoints and 31) = 0 then
        Write(#13, NPoints / MaxPoints:6:2);

    End; { For }

    { update the Y position and index }
    YVal := YVal + GridYStep;
    Y := Y + 1;

    { check for a key press }
    If KeyPressed then
      Ch := GetKey

  { continue until all points evaluated or until ESC pressed }
  Until (Y > YGridSize) or (Ch = #27)

End; { Procedure ComputeGrid }

{---------------------------------------------------------------------------}

End. { Unit GridFunc }
