Unit MathUnit;
{
  Peculiar math functions (Power, Factorial, Bessel, Gamma,
                           Quadratic escape-time fractal).
}

Interface
Uses
  Vectors;


Function Power(Base, Exponent : Float) : Float;
Function Factorial(X : Float) : Float;
Function Gamma(XX : Float) : Float;
Function Bessel(X, N : Float) : Float;


Implementation
Uses
  FDecls;    { for misc data }

{---------------------------------------------------------------------------}

Function Power(Base, Exponent : Float) : Float;
{
  Number to a power computation
}
Var
  I : Integer;
Begin
  If Base = 0.0 then
    Power := 0.0
  Else If Exponent = 0.0 then
    Power := 1.0
  Else
    Power := exp(Exponent * ln(abs(Base)))
End; { Function Power }

{---------------------------------------------------------------------------}

Function Factorial(X : Float) : Float;
{
  Factorial using Stirling's approximation with correction factor.
  Taken from the book Turbo Algorithms.
}
Const { table of factorial values < 32 }
  FacTable : Array[0..32] of Float = ( 1.0,
                                       1.0,
                                       2.0,
                                       6.0,
                                      24.0,
                                     120.0,
                                     720.0,
                                    5040.0,
                                   40320.0,
                                  362880.0,
                                 3628800.0,
                                39916800.0,
                               479001600.0,
                              6227020800.0,
                             87178291200.0,
                           1307674368000.0,
                          20922789888000.0,
                         355687428096000.0,
                        6402373705728000.0,
                      121645100408832000.0,
                     2432902008176640000.0,
                    51090942171709440000.0,
                  1124000727777607680000.0,
                 25852016738884976600000.0,
                620448401733239439000000.0,
              15511210043330986000000000.0,
             403291461126605636000000000.0,
           10888869450418352200000000000.0,
          304888344611713860000000000000.0,
         8841761993739701960000000000000.0,
       265252859812191059000000000000000.0,
      8222838654177922820000000000000000.0,
    263130836933693530000000000000000000.0);
Begin

  If (X < 0.0) or (X > 500.0) then
    If MathCheck then
      Abort('Invalid factorial value')
    Else
      Factorial := MathErrValue
  Else If X <= 32.0 then
    Factorial := FacTable[Trunc(X)]
  Else begin
    X := Trunc(X);
    Factorial := sqrt(2*Pi*X) * exp(X*ln(X)-X) *
      (1 + 1/(12*X) + 1/(288*sqr(X)) -
           139/(51840*sqr(X)*X) - 571/(2488320*sqr(sqr(X))))
  End { Else }

End; { Function Factorial }

{---------------------------------------------------------------------------}

Function GammaLn(XX : Float) : Float;
{
  Natural log of gamma function.
  Modified from the C function given in Numerical Recipes in C -
  Press, Flannery Teukolsky, Vetterling.
  Page 174.
}
Const
  cof : Array[0..5] of Float = (
    76.18009173, -86.50532033,     24.01409822,
    -1.231739516,  0.120858003E-2, -0.536382E-5
  );
Var
  x, tmp, ser : Float;
  j : Integer;
Begin

  x := xx - 1.0;
  tmp := x + 5.5;
  tmp := tmp - (x + 0.5) * ln(tmp);
  ser := 1.0;
  For j := 0 to 5 do begin
    x := x + 1.0;
    ser := ser + cof[j] / x;
  End; { For }
  GammaLn := -tmp + ln(2.50662827465 * ser);

End; { Function GammaLn }

{---------------------------------------------------------------------------}

Function Gamma(XX : Float) : Float;
{
  Gamma function.
}
Begin
  Gamma := exp(GammaLn(XX));
End; { Function Gamma }

{---------------------------------------------------------------------------}

Function Bessel(X, N : Float) : Float;
{
  Bessel function
}
Const
  MaxTerms = 10;
  precision = 1e-8;
Var
  k : Integer;
  x2,
  oterm,
  Acc,
  Sign : Float;
  done : Boolean;


  Function  Bessj0(X : Float) : Float;
  {
    Returns the Bessel Function J0 for any real X
  }
  Var
    ax, z, xx, y, ans, ans1, ans2 : Float;
  Begin

    ax := abs(x);
    If (ax < 8.0) then begin
      y := x * x;
      ans1 :=       57568490574.0 +
              y * (-13362590354.0 +
              y * ( 651619640.7   +
              y * (-11214424.18   +
              y * ( 77392.33017   +
              y * (-184.9052456)))));
      ans2 :=       57568490411.0 +
              y * ( 1029532985.0  +
              y * ( 9494680.718   +
              y * ( 59272.64853   +
              y * ( 267.8532712   +
              y * ( 1.000000000)))));
      Bessj0 := ans1 / ans2
    End { If }
    Else begin
      z := 8.0 / ax;
      y := z * z;
      xx := ax - 0.78398164;
      ans1 :=       1.0000000000    +
              y * (-0.1098628270E-2 +
              y * ( 0.2734510407E-4 +
              y * (-0.2073370639E-5 +
              y * ( 0.2093887211E-6))));
      ans2 :=      -0.1562499995E-1 +
              y * ( 0.1430488765E-3 +
              y * (-0.6911147651E-5 +
              y * ( 0.7621095161E-6 +
              y * (-0.9349351520E-7))));
      Bessj0 := sqrt(0.636619772 / ax) * cos(xx) * ans1 - z * sin(xx) * ans2
    End { Else }

  End; { Function Bessj0 }


Begin

  x2 := x / 2.0;
  Sign := 1.0;
  k := 0;
  done := False;
  Acc := 0.0;
  oterm := Acc + 3.319273914076;

  Repeat
    Acc := Acc + Sign * power(x2,N + 2.0 * k) /
                 (Factorial(k) * Gamma(N + k + 1.0));
    done := abs(oterm - Acc) <= precision;
    Sign := -Sign;
    oterm := Acc;
    Inc(k)
  Until (k > MaxTerms) or done;

  Bessel := Acc;

End; { Function Bessel }

{---------------------------------------------------------------------------}

Function Noise(X, Y, Z : Float) : Float;
{
  Generate a nice 3-D noise value.
}
Begin
  Noise := 0.0;
End; { Function Noise }

{---------------------------------------------------------------------------}

End. { Unit MathUnit }
