Unit Plasma;
{
  Computes a plasma-type fractal surface using the whole surface.
  The result ranges from -1 to +1.
}

Interface

Procedure PlasmaCalc;

Implementation

Uses
  FDecls,
  Vectors;

{---------------------------------------------------------------------------}

const
  RandFactor = 125;

Procedure PlasmaHLineInterp(X1, X2, Y : Integer);
{
  Interpolate along a horizontal line
}
Var
  X : Integer;
  q : Float;
begin

  { If there's still something we can interpolate along }
  If X2 - X1 >= 2 then begin

    { determine the center point }
    X := (X1 + X2 + 1) div 2;

    { take the average of the line ends plus a random factor related to the
      scale (X2 - X1) and an independent random factor (4) }
    q := (Grid[X1]^[Y] + Grid[X2]^[Y]) / 2.0 +
         (2.0 * random - 1.0) * (X2 - X1) / RandFactor +
         (2.0 * random - 1.0) * (4      ) / RandFactor;

    { bound the result to +/- 1.0 }
    If q < -1.0 then
      q := -1.0
    Else If q > 1.0 then
      q := 1.0;

    { set the data element into the surface array }
    Grid[X]^[Y] := q;

    { recursively calculate the left side }
    PlasmaHLineInterp(X1, X, Y);

    { recursively calculate the right side }
    PlasmaHLineInterp(X, X2, Y)

  End { If }

End; { PlasmaHLineInterp }

{---------------------------------------------------------------------------}

Procedure PlasmaVLineInterp(Y1,Y2,X: Integer);
{
  Interpolate along a vertical slice of the surface
}
Var
  Y : Integer;
  q : Float;
begin

  { If there's still something we can interpolate along }
  If Y2 - Y1 >= 2 then begin

    { determine the center point }
    Y := (Y1 + Y2 + 1) div 2;

    { take the average of the line ends plus a random factor related to the
      scale (Y2 - Y1) and an independent random factor (4) }
    q := (Grid[X]^[Y1] + Grid[X]^[Y2]) / 2 +
         (2 * random - 1) * (Y2 - Y1) / RandFactor +
         (2 * random - 1) * (4      ) / RandFactor;

    { bound the result to +/- 1.0 }
    If q < -1 then
      q := -1
    Else If q > 1 then
      q := 1;

    { set the data element into the surface array }
    Grid[X]^[Y] := q;

    { recursively calculate the top half }
    PlasmaVLineInterp(Y1, Y, X);

    { recursively calculate the bottom half }
    PlasmaVLineInterp(Y, Y2, X)

  End { If }

End; { PlasmaVLineInterp }

{---------------------------------------------------------------------------}

Procedure PlasmaSubDivide(X1, Y1, X2, Y2 : Integer);
{
}
Var
  X, Y : Integer;
begin 

  { If there's still something to interpolate }
  If (X2 - X1 >= 2) and (Y2 - Y1 >= 2) then begin

    { If the horizontal distance is greater than the vertical distance }
    If X2 - X1 > Y2 - Y1 then begin

      { determine the center of the horizontal line }
      X := (X1 + X2 + 1) div 2;

      { interpolate the column at that point }
      PlasmaVLineInterp(Y1, Y2, X);

      { interpolate the left half of the surface }
      PlasmaSubDivide(X1, Y1,  X, Y2);

      { interpolate the right half of the surface }
      PlasmaSubDivide( X, Y1, X2, Y2);

    End { If }
    Else begin

      { determine the center of the vertical line }
      Y := (Y1 + Y2 + 1) div 2;

      { interpolate the row at that point }
      PlasmaHLineInterp(X1, X2, Y);

      { interpolate the top half of the surface }
      PlasmaSubDivide(X1, Y1, X2,  Y);

      { interpolate the bottom half of the surface }
      PlasmaSubDivide(X1,  Y, X2, Y2);

    End { Else }

  End { If }

End; { PlasmaSubDivide }

{---------------------------------------------------------------------------}

Procedure PlasmaCalc;
Var
  X, Y,
  I, J, N: Integer;
  A : Float;

Begin

  { init corners }
  Grid[        0]^[        0] := 2 * random - 1;
  Grid[XGridSize]^[        0] := 2 * random - 1;
  Grid[        0]^[YGridSize] := 2 * random - 1;
  Grid[XGridSize]^[YGridSize] := 2 * random - 1;

  { interpolate from corners to get outline of screen }
  PlasmaHLineInterp(0, XGridSize,         0);
  PlasmaVLineInterp(0, YGridSize, XGridSize);
  PlasmaHLineInterp(0, XGridSize, YGridSize);
  PlasmaVLineInterp(0, YGridSize,         0);

  { do interior of screen }
  PlasmaSubDivide(0, 0, XGridSize, YGridSize);

  { do terrain adjustments }
  If (PosPlasmaExponent <> 0.0) then begin
    For X := 0 to XGridSize do begin
      For Y := 0 to YGridSize do begin
        If Grid[X]^[Y] > 0 then
          Grid[X]^[Y] := exp(PosPlasmaExponent * ln( Grid[X]^[Y]))
      End { For }
    End { For }
  End; { If }
  If (NegPlasmaExponent <> 0.0) then begin
    NegPlasmaExponent := 1.0 / NegPlasmaExponent;
    For X := 0 to XGridSize do begin
      For Y := 0 to YGridSize do begin
        If Grid[X]^[Y] < 0 then
          Grid[X]^[Y] := -exp(NegPlasmaExponent * ln(-Grid[X]^[Y]))
      End { For }
    End { For }
  End; { If }

  { smooth the surface? }
  If (PlasmaSmooth > 0) and (PlasmaSmooth < XGridSize / 4) then begin
    N := Round(PlasmaSmooth);
    For X := N to XGridSize - N do begin
      For Y := N to YGridSize - N do begin
        A := 0.0;
        For I := -N to N do begin
          For J := -N to N do begin
            A := A + Grid[X + I]^[Y + J];
          End; { For }
        End; { For }
        Grid[X]^[Y] := A;
      End; { For }
    End; { For }
  End; { If }

End; { PlasmaCalc }

End.

{---------------------------------------------------------------------------}
