Unit QuadFrac;
{
  Unit for computation of quadratic fractals (Mandelbrot and Julia sets,
  among others).
}

Interface
Uses
  Vectors;

Const
  AbsMaxIterations = 1024;           { used in fractal computations }
  MaxRadius     : Float = 10000;
  MaxIterations : Word  = 256;
  RealJuliaParm : Float = 0.0;
  ImagJuliaParm : Float = 1.0;

Function Potential(X, Y, RealParm, ImagParm : Float) : Float;


Implementation

{ variables used by the Potential routine }
Var
  x2, y2       : Float;                 { complex accumulator parts squared }
  Iteration    : Word;                  { current iteration count }
  Done         : Boolean;               { value stream entered a cycle?? }
  xCheck,                               { hold area for cycle checking }
  yCheck       : Array[0..AbsMaxIterations] of Float;

Function Potential(X, Y, RealParm, ImagParm : Float) : Float;
{
  Return potential of quadratic Mandelbrot or Julia set at current point.
  Also used to do the QUADFRAC routine.
}
Const
  ln2  = 0.693147180559945309;
  ln10 = 2.302585092994045680;

Begin

  { start with no iterations (haven't done anything yet) }
  { compute x*x and y*y }
  x2 := sqr(x);
  y2 := sqr(y);

  Iteration := 1;
  Repeat

    { evaluate formula z*z + c }
    y := 2 * x * y + ImagParm;
    x := x2 - y2   + RealParm;

    { compute x*x and y*y }
    x2 := sqr(x);
    y2 := sqr(y);

    { count iteration }
    Inc(Iteration);

    { enter the result into cycle check table }
    xCheck[Iteration] := x;
    yCheck[Iteration] := y;

    Done := (Iteration >= MaxIterations) or
             ((xCheck[Iteration shr 1] = x) and
              (yCheck[Iteration shr 1] = y))

  Until (x2 + y2 > MaxRadius) or               { outside set }
         Done;                                { inside set  }

  If not Done then
    Potential := 0.5 * (ln(x2 + y2) / ln10) / exp(Iteration * ln2)
  Else
    Potential := 0.0

End; { Function Potential }

{---------------------------------------------------------------------------}

Begin

  { clear cycle check data }
  FillChar(xCheck, SizeOf(xCheck), 0);
  FillChar(yCheck, SizeOf(yCheck), 0);

End.