Unit Vectors;

(****************************************************************************
**                                                                         **
**    Routines for easier operations upon 3D vectors and 3D cartesian      **
**    coordinates.  Also makes a dandy way to specify a faster (and/or     **
**    accurate) floating-point number type (Float) for the program.        **
**                                                                         **
****************************************************************************)

Interface

Const
  RealSmall = {$IFOPT N+} 1E-1024 {$ELSE} 1E-18 {$ENDIF};

Type
  Float = {$IFOPT N+} Extended {$ELSE} Real {$ENDIF};

  { note that since the definitions of Vector and Cartesian are physically
    identical, Vector(n) = Cartesian(n). }

  Vector = Record       { 3D vector }
    i,j,k : Float
  End; { Record }

  Cartesian = Record    { 3D coordinate }
    x,y,z : Float
  End; { Record }

{
  vector and cartesian structures are given as VAR parameters where possible.
  Using VAR specifiers saves some time but may lead to ambiguities (and
  erroneous results) with certain routines.
}

Procedure MakeVector(X,Y,Z : Float; Var Result : Vector);
Procedure MakeUnitVector(X,Y,Z : Float; Var Result : Vector);
Procedure Unitize(var v : Vector);
Function  GetDotProduct(var Vec1,Vec2 : Vector) : Float;
Procedure GetCrossProduct(A,B : Vector; Var AxB : Vector);
Function  Magnitude(var A : Vector) : Float;
Function  SqrMagnitude(var A : Vector) : Float;
Procedure VectorBetween(var P1,P2 : Cartesian; Var Result : Vector);
Function  DistBetween(var P1,P2 : Cartesian) : Float;
Function  SqrDistBetween(var P1,P2 : Cartesian) : Float;

Implementation

{---------------------------------------------------------------------------}

Procedure MakeVector;
{
  Make a vector from given XYZ values
}
Begin
  With Result do begin
    i := X;
    j := Y;
    k := Z
  End { With }
End; { Procedure MakeVector }

{---------------------------------------------------------------------------}

Procedure MakeUnitVector;
{
  Manufacture a unit vector
}
Var
  Len : Float;
Begin
  With Result do begin

    { make vector }
    i := X;
    j := Y;
    k := Z;

    { convert to a unit vector }
    Len := sqrt(sqr(i) + sqr(j) + sqr(k));
    If Len < RealSmall then begin
      i := 0.0;
      j := 0.0;
      k := 0.0
    End { If }
    Else begin
      i := i / Len;
      j := j / Len;
      k := k / Len
    End { Else }

  End { With }

End; { Procedure MakeVector }

{---------------------------------------------------------------------------}

Procedure Unitize;
{
  Convert vector to a unit vector
}
Var
  Len : Float;
Begin

  With v do begin

    { compute normalization factor }
    Len := Sqrt(sqr(i) + sqr(j) + sqr(k));

    { prevent underflow errors while computing result }
    If Len < RealSmall then begin
      i := 0.0;
      j := 0.0;
      k := 0.0
    End { If }
    Else begin
      i := i / Len;
      j := j / Len;
      k := k / Len
    End { Else }

  End { With }

End; { Procedure Unitize }

{---------------------------------------------------------------------------}

Function GetDotProduct;
{
  Return Vec1 . Vec2 (not normalized)
}
Begin
  GetDotProduct := Vec1.i*Vec2.i + Vec1.j*Vec2.j + Vec1.k*Vec2.k
End; { Function GetDotProduct }

{---------------------------------------------------------------------------}

Procedure GetCrossProduct;
{
  Return A x B
}
Begin

  AxB.i := A.j*B.k - A.k*B.j;
  AxB.j := A.k*B.i - A.i*B.k;
  AxB.k := A.i*B.j - A.j*B.i

End; { Procedure GetCrossProduct }

{---------------------------------------------------------------------------}

Function Magnitude;
{
  Compute a vector's magnitude
}
Begin

  With A do
    Magnitude := sqrt(sqr(i) + sqr(j) + sqr(k))

End; { Function Magnitude }

{---------------------------------------------------------------------------}

Function SqrMagnitude;
{
  Compute the square of a vector's magnitude
}
Begin

  With A do
    SqrMagnitude := sqr(i) + sqr(j) + sqr(k)

End; { Function Magnitude }

{---------------------------------------------------------------------------}

Procedure VectorBetween;
{
  Manufacture a vector starting at P2 and extending to P2
}
Begin

  Result.i := P2.x - P1.x;
  Result.j := P2.y - P1.y;
  Result.k := P2.z - P1.z

End; { Procedure VectorBetween }

{---------------------------------------------------------------------------}

Function DistBetween;
{
  Compute distance between two points
}
Begin

  DistBetween := Sqrt(sqr(P1.x - P2.x) + sqr(P1.y - P2.y) + sqr(P1.z - P2.z))

End; { Function DistBetween }

{---------------------------------------------------------------------------}

Function SqrDistBetween;
{
  Compute square of distance between two points
}
Begin

  SqrDistBetween := sqr(P1.x - P2.x) + sqr(P1.y - P2.y) + sqr(P1.z - P2.z)

End; { Function SqrDistBetween }

{---------------------------------------------------------------------------}

End. { Unit Vectors }
